(function() {
	'use strict';

	/**
	 * matches() pollyfil
	 * @see https://developer.mozilla.org/en-US/docs/Web/API/Element/closest#Polyfill
	 */
	if ( ! Element.prototype.matches ) {
		Element.prototype.matches = Element.prototype.msMatchesSelector || Element.prototype.webkitMatchesSelector;
	}

	/**
	 * closest() pollyfil
	 * @see https://developer.mozilla.org/en-US/docs/Web/API/Element/closest#Polyfill
	 */
	if ( ! Element.prototype.closest ) {
		Element.prototype.closest = function( s ) {
			var el = this;
			if ( ! document.documentElement.contains( el ) ) {
				return null;
			}
			do {
				if ( el.matches( s ) ) {
					return el;
				}
				el = el.parentElement || el.parentNode;
			} while ( el !== null && el.nodeType === 1 ); 
			return null;
		};
	}

	window.wengdoHelper = {
		/**
		 * Helper function to get element's offset.
		 */
		getOffset: function( $el ) {
			if ( $el instanceof HTMLElement ) {
				var rect = $el.getBoundingClientRect();

				return {
					top: rect.top + window.pageYOffset,
					left: rect.left + window.pageXOffset,
				}
			}

			return {
				top: null,
				left: null,
			};
		},

		/**
		 * Helper function to check if element's visible or not.
		 */
		isVisible: function( $el ) {
			return $el.offsetWidth > 0 && $el.offsetHeight > 0;
		},

		/**
		 * Function to check RTL
		 */
		isRTL: function() {
			return document.body.classList.contains( 'rtl' );
		},

		/**
		 * Function to hide an element using slideUp animation.
		 *
		 * source: https://w3bits.com/javascript-slidetoggle/
		 */
		slideUp: function( target, duration ) {
			if ( ! target ) return;

			duration = ( typeof duration !== 'undefined' ) ? duration : 250;

			target.style.transitionProperty = 'height, margin, padding';
			target.style.transitionDuration = duration + 'ms';
			target.style.height = target.offsetHeight + 'px';
			target.offsetHeight;
			target.style.overflow = 'hidden';
			target.style.height = 0;
			target.style.paddingTop = 0;
			target.style.paddingBottom = 0;
			target.style.marginTop = 0;
			target.style.marginBottom = 0;

			window.setTimeout( function() {
				target.removeAttribute( 'style' );
			}, duration );
		},

		/**
		 * Function to show an element using slideDown animation.
		 *
		 * source: https://w3bits.com/javascript-slidetoggle/
		 */
		slideDown: function( target, duration ) {
			if ( ! target ) return;

			duration = ( typeof duration !== 'undefined' ) ? duration : 250;

			target.style.removeProperty( 'display' );

			var display = window.getComputedStyle( target ).display;
			if ( display === 'none' ) {
				display = 'block';
			}
			target.style.display = display;

			var height = target.offsetHeight;

			target.style.overflow = 'hidden';
			target.style.height = 0;
			target.style.paddingTop = 0;
			target.style.paddingBottom = 0;
			target.style.marginTop = 0;
			target.style.marginBottom = 0;
			target.offsetHeight;
			target.style.transitionProperty = 'height, margin, padding';
			target.style.transitionDuration = duration + 'ms';
			target.style.height = height + 'px';
			target.style.removeProperty( 'padding-top' );
			target.style.removeProperty( 'padding-bottom' );
			target.style.removeProperty( 'margin-top' );
			target.style.removeProperty( 'margin-bottom' );

			window.setTimeout( function() {
				target.style.removeProperty( 'height' );
				target.style.removeProperty( 'overflow' );
				target.style.removeProperty( 'transition-duration' );
				target.style.removeProperty( 'transition-property' );
			}, duration );
		},

		/**
		 * Function to toggle visibility of an element using slideUp or SlideDown animation.
		 *
		 * source: https://w3bits.com/javascript-slidetoggle/
		 */
		slideToggle: function( target, duration ) {
			if ( ! target ) return;

			duration = ( typeof duration !== 'undefined' ) ? duration : 250;

			if ( window.getComputedStyle( target ).display === 'none' ) {
				return slideDown( target, duration );
			} else {
				return slideUp( target, duration );
			}
		},
	}

	window.wengdo = {

		/**
		 * Function to init different style of focused element on keyboard users and mouse users.
		 */
		initKeyboardAndMouseFocus: function() {
			document.body.addEventListener( 'keydown', function( e ) {
				document.body.classList.add( 'using-keyboard' );
			}, false );

			document.body.addEventListener( 'mousedown', function( e ) {
				document.body.classList.remove( 'using-keyboard' );
			}, false );
		},

		/**
		 * Function to init edge sub menu detection script.
		 */
		initDropdownMenuReposition: function() {
			var prop = window.wengdoHelper.isRTL() ? 'right' : 'left';

			var calculateSubMenuEdge = function() {

				var $submenus = Array.prototype.slice.call( document.querySelectorAll( '.wengdo-header-section .menu > * > .sub-menu' ) );
				$submenus.forEach(function( $submenu ) {
					var $section = $submenu.closest( '.wengdo-header-section' ),
					    $container = $section.classList.contains( 'wengdo-section-contained' ) ? $section.querySelector( '.wengdo-section-inner' ) : $submenu.closest( '.wengdo-wrapper' );

					// Reset inline styling.
					$submenu.classList.remove( 'wengdo-sub-menu-edge' );
					$submenu.style[ prop ] = '';

					$submenu.style.maxWidth = $container.offsetWidth + 'px';

					var containerEdge = $container.getBoundingClientRect().left + ( window.wengdoHelper.isRTL() ? 0 : $container.getBoundingClientRect().width ),
						submenuEdge = $submenu.getBoundingClientRect().left + ( window.wengdoHelper.isRTL() ? 0 : $submenu.getBoundingClientRect().width ),
						isSubmenuOverflow = window.wengdoHelper.isRTL() ? submenuEdge < containerEdge : submenuEdge > containerEdge;

					// Apply class and left position.
					if ( isSubmenuOverflow ) {
						$submenu.classList.add( 'wengdo-sub-menu-edge' );
						$submenu.style[ prop ] = -1 * Math.abs( containerEdge - submenuEdge ).toString() + 'px';
					}

					// Apply vertical max-height.
					$submenu.style.maxHeight = ( window.innerHeight - $submenu.getBoundingClientRect().top ) + 'px';

					// Iterate to 2nd & higher level submenu.
					var $subsubmenus = Array.prototype.slice.call( $submenu.querySelectorAll( '.sub-menu' ) );
					$subsubmenus.forEach(function( $subsubmenu ) {
						var subsubmenuEdge = $subsubmenu.getBoundingClientRect().left + ( window.wengdoHelper.isRTL() ? 0 : $subsubmenu.getBoundingClientRect().width ),
							isSubsubmenuOverflow = window.wengdoHelper.isRTL() ? subsubmenuEdge < containerEdge : subsubmenuEdge > containerEdge;

						// Apply class and left position.
						if ( isSubsubmenuOverflow ) {
							$subsubmenu.classList.add( 'wengdo-sub-menu-right' );
						}

						// Apply vertical max-height.
						$subsubmenu.style.maxHeight = ( window.innerHeight - $subsubmenu.getBoundingClientRect().top ) + 'px';
					});
				});
			}

			var timeout;
			window.addEventListener( 'resize', function() {
				clearTimeout( timeout );
				timeout = setTimeout( calculateSubMenuEdge, 500 );
			}, false );
			calculateSubMenuEdge();
		},

		/**
		 * Function to init hover menu.
		 */
		initMenuAccessibility: function() {

			/**
			 * Accesibility using tab button
			 * ref: https://github.com/wpaccessibility/a11ythemepatterns/blob/master/dropdown-menus/vanilla-js/js/dropdown.js
			 */
			var handleMenuFocusUsingKeyboard = function( e ) {
				// Fix Firefox issue.
				if ( document === e.target ) return;

				// Check target element.
				var $this = e.target.closest( '.wengdo-hover-menu .menu-item > a' );
				if ( ! $this ) return;

				var $menu = $this.closest( '.wengdo-hover-menu' ),
				    $current = this;

				while ( $current !== $menu ) {
					if ( $current.classList.contains( 'menu-item' ) ) {
						if ( $current.classList.contains( 'focus' ) ) {
							$current.classList.remove( 'focus' );
						} else {
							$current.classList.add( 'focus' );
						}
					}
					$current = $current.parentElement;
				}
			}
			document.addEventListener( 'focus', handleMenuFocusUsingKeyboard, false );
			document.addEventListener( 'blur', handleMenuFocusUsingKeyboard, false );

			/**
			 * Accesibility using arrow nav buttons
			 * ref: https://github.com/wpaccessibility/a11ythemepatterns/blob/master/menu-keyboard-arrow-nav/vanilla-js/js/navigation.js
			 */
			var handleMenuNavigationUsingKeyboard = function( e ) {
				// Check target element.
				var $this = e.target.closest( '.wengdo-hover-menu .menu-item > a' );
				if ( ! $this ) return;

				var key = e.which || e.keyCode;

				// left key
				if ( 37 === key ) {
					e.preventDefault();

					if ( $this.parentElement.previousElementSibling ) {
						$this.parentElement.previousElementSibling.firstElementChild.focus();
					}
				}
				// right key
				else if ( 39 === key ) {
					e.preventDefault();

					if ( $this.parentElement.nextElementSibling ) {
						$this.parentElement.nextElementSibling.firstElementChild.focus();
					}
				}
				// down key
				else if ( 40 === key ) {
					e.preventDefault();

					if ( $this.nextElementSibling ) {
						$this.nextElementSibling.firstElementChild.firstElementChild.focus();
					}
					else if ( $this.parentElement.nextElementSibling ) {
						$this.parentElement.nextElementSibling.firstElementChild.focus();
					}
				}
				// up key
				else if ( 38 === key ) {
					e.preventDefault();

					if ( $this.parentElement.previousElementSibling ) {
						$this.parentElement.previousElementSibling.firstElementChild.focus();
					}
					else if ( $this.parentElement.parentElement.previousElementSibling ) {
						$this.parentElement.parentElement.previousElementSibling.focus();
					}
				}
			}
			document.addEventListener( 'keydown', handleMenuNavigationUsingKeyboard, false );
		},

		/**
		 * Function to init double tap menu on mobile devices.
		 */
		initDoubleTapMobileMenu: function() {
			/**
			 * Mobile Touch friendly
			 */
			var handleMenuOnMobile = function( e ) {
				// Check target element.
				var $this = e.target.closest( '.wengdo-hover-menu .menu-item > a' );
				if ( ! $this ) return;

				var $menuItem = $this.parentElement;

				// Only enable double tap on menu item that has sub menu and it's not a empty hash link.
				if ( $menuItem.classList.contains( 'menu-item-has-children' ) ) {
					if ( $this !== document.activeElement ) {
						document.activeElement.blur();
						$this.focus();

						e.preventDefault();
					}
				}
			}
			document.addEventListener( 'touchend', handleMenuOnMobile, false );
		},

		/**
		 * Function to init toggle menu.
		 */
		initClickToggleDropdownMenu: function() {
			/**
			 * Toggle Handler
			 */
			var handleSubMenuToggle = function( e ) {
				// Check target element.
				var $this = e.target.closest( '.wengdo-header-section .wengdo-toggle-menu .wengdo-sub-menu-toggle' );
				if ( ! $this ) return;
				
				e.preventDefault();

				var $header = document.getElementById( 'masthead' ),
				    $menuItem = $this.parentElement;

				// Menu item already has "focus" class, so collapses itself.
				if ( $menuItem.classList.contains( 'focus' ) ) {
					$menuItem.classList.remove( 'focus' );
					$this.setAttribute( 'aria-expanded', false );
				}
				// Menu item doesn't have "focus" class yet, so collapses other focused menu items found in the header and focuses this menu item.
				else {
					var $focusedMenuItems = Array.prototype.slice.call( $header.querySelectorAll( '.menu-item.focus' ) );
					$focusedMenuItems.forEach(function( $focusedMenuItem ) {
						$focusedMenuItem.classList.remove( 'focus' );
					});

					$menuItem.classList.add( 'focus' );
					$this.setAttribute( 'aria-expanded', true );

					// Move focus to search bar (if exists).
					var $searchBar = $menuItem.querySelector( '.search-field' );
					if ( $searchBar ) {
						setTimeout(function() {
							$searchBar.focus();
						}, 300 );
					}
				}
			}
			document.addEventListener( 'click', handleSubMenuToggle, false );
			document.addEventListener( 'touchend', handleSubMenuToggle, false );

			/**
			 * Close Handler
			 */
			var handleSubMenuClose = function( e ) {
				// Make sure click event doesn't happen inside the menu item's scope.
				if ( ! e.target.closest( '.wengdo-header-section .wengdo-toggle-menu' ) ) {
					var $header = document.getElementById( 'masthead' ),
					    $focusedMenuItems;

					if ( $header ) {
						var $focusedMenuItems = Array.prototype.slice.call( $header.querySelectorAll( '.wengdo-toggle-menu .menu-item.focus' ) );
						$focusedMenuItems.forEach(function( $focusedMenuItem ) {
							$focusedMenuItem.classList.remove( 'focus' );
							$clickedToggle.setAttribute( 'aria-expanded', false );
						});
					}
				}
			};
			document.addEventListener( 'click', handleSubMenuClose, false );
			document.addEventListener( 'touchend', handleSubMenuClose, false );
		},

		/**
		 * Function to init mobile menu.
		 */
		initAccordionMenu: function() {
			/**
			 * Toggle Handler
			 */
			var handleAccordionMenuToggle = function( e ) {
				// Check target element.
				var $this = e.target.closest( '.wengdo-header-section-vertical .wengdo-toggle-menu .wengdo-sub-menu-toggle' );
				if ( ! $this ) return;
				
				e.preventDefault();

				var $menuItem = $this.parentElement,
				    $subMenu = $menuItem.querySelector( '.sub-menu' );

				// Menu item already has "focus" class, so collapses itself and all menu items inside.
				if ( $menuItem.classList.contains( 'focus' ) ) {
					window.wengdoHelper.slideUp( $subMenu );
					$menuItem.classList.remove( 'focus' );

					var $insideMenuItems = Array.prototype.slice.call( $menuItem.querySelectorAll( '.menu-item.focus' ) );
					$insideMenuItems.forEach(function( $insideMenuItem ) {
						window.wengdoHelper.slideUp( $insideMenuItem.querySelector( '.sub-menu' ) );
						$insideMenuItem.classList.remove( 'focus' );
					});
				}
				// Menu item doesn't have "focus" class yet, so collapses all focused siblings and focuses this menu item.
				else {
					var $siblingMenuItems = Array.prototype.slice.call( $menuItem.parentElement.querySelectorAll( '.menu-item.focus' ) );
					$siblingMenuItems.forEach(function( $siblingMenuItem ) {
						window.wengdoHelper.slideUp( $siblingMenuItem.querySelector( '.sub-menu' ) );
						$siblingMenuItem.classList.remove( 'focus' );
					});

					window.wengdoHelper.slideDown( $subMenu );
					$menuItem.classList.add( 'focus' );
				}
			}
			document.addEventListener( 'click', handleAccordionMenuToggle, false );
			document.addEventListener( 'touchend', handleAccordionMenuToggle, false );

			/**
			 * Empty Hash Link Handler
			 */
			var handleAccordionMenuEmptyHashLink = function( e ) {
				// Check target element.
				var $this = e.target.closest( '.wengdo-header-section-vertical .wengdo-toggle-menu .menu-item-has-children > .wengdo-menu-item-link[href="#"]' );
				if ( ! $this ) return;

				e.preventDefault();

				var $menuItem = $this.parentElement,
				    $toggle = $menuItem.querySelector( '.wengdo-sub-menu-toggle' );

				// If an empty hash link is clicked, trigger the toggle click event.
				// ref: https://gomakethings.com/how-to-simulate-a-click-event-with-javascript/
				$toggle.click();
			}
			document.addEventListener( 'click', handleAccordionMenuEmptyHashLink, false );
			document.addEventListener( 'touched', handleAccordionMenuEmptyHashLink, false );
		},

		/**
		 * Function to init page popup toggle.
		 */
		initGlobalPopup: function() {
			var $clickedToggle = null;

			var deactivatePopup = function( device ) {
				var $activePopups = Array.prototype.slice.call( document.querySelectorAll( '.wengdo-popup-active' + ( undefined !== device ? '.wengdo-hide-on-' + device : '' ) ) );

				$activePopups.forEach(function( $activePopup ) {
					// Deactivate popup.
					$clickedToggle.classList.remove( 'wengdo-popup-toggle-active' );
					$clickedToggle.setAttribute( 'aria-expanded', false );
					$activePopup.classList.remove( 'wengdo-popup-active' );
					document.body.classList.remove( 'wengdo-has-popup-active' );

					// Back current focus to the toggle.
					$activePopup.removeAttribute( 'tabindex' );
					if ( document.body.classList.contains( 'using-keyboard' ) ) {
						$clickedToggle.focus();
					}
				});
			}

			// Show / hide popup when the toggle is clicked.
			var handlePopupToggle = function( e ) {
				// Check target element.
				var $this = e.target.closest( '.wengdo-popup-toggle' );
				if ( ! $this ) return;

				e.preventDefault();
				
				var $target = document.querySelector( '#' + $this.getAttribute( 'data-target' ) );

				// Abort if no popup target found.
				if ( ! $target ) return;

				if ( $target.classList.contains( 'wengdo-popup-active' ) ) {
					deactivatePopup();
				} else {
					// Activate popup.
					$this.classList.add( 'wengdo-popup-toggle-active' );
					$this.setAttribute( 'aria-expanded', true );
					$target.classList.add( 'wengdo-popup-active' );
					document.body.classList.add( 'wengdo-has-popup-active' );

					// Put focus on popup.
					setTimeout(function() {
						$target.setAttribute( 'tabindex', 0 );
						$target.focus();
					}, 300 );

					// Save this toggle for putting back focus when popup is deactivated.
					$clickedToggle = $this;
				}
			}
			document.addEventListener( 'click', handlePopupToggle, false );
			document.addEventListener( 'touchend', handlePopupToggle, false );

			// Close popup when any of ".wengdo-popup-close" element is clicked.
			var handlePopupClose = function( e ) {
				// Check target element.
				if ( ! e.target.closest( '.wengdo-popup-close' ) ) return;

				e.preventDefault();

				deactivatePopup();
			}
			document.addEventListener( 'click', handlePopupClose, false );
			document.addEventListener( 'touchend', handlePopupClose, false );

			// Close popup using "escape" keyboard button.
			var handlePopupEscape = function( e ) {
				var key = e.which || e.keyCode;

				if ( document.body.classList.contains( 'wengdo-has-popup-active' ) && 27 === key ) {
					deactivatePopup();
				}
			}
			document.addEventListener( 'keydown', handlePopupEscape, false );

			// When window resize, close Active Popups based on their responsive visibility classes.
			var handleResponsiveVisibility = function( e ) {
				if ( document.body.classList.contains( 'wengdo-has-popup-active' ) ) {
					var device = 'mobile';

					if ( wengdoConfig.breakpoints.mobile <= window.innerWidth ) {
						device = 'tablet';
					}

					if ( wengdoConfig.breakpoints.desktop <= window.innerWidth ) {
						device = 'desktop';
					}

					deactivatePopup( device );
				}
			}
			window.addEventListener( 'resize', handleResponsiveVisibility, false );

			// Close popup if any hash link is clicked.
			var handleHashLinkInsidePopup = function( e ) {
				// Check target element.
				if ( ! e.target.closest( '.wengdo-popup a' ) ) return;

				var $link = e.target.closest( 'a' );

				// Check if the link is a hash link.
				if ( '' !== $link.hash ) {
					var pageURL = ( window.location.hostname + '/' + window.location.pathname ).replace( '/\/$/', '' ),
					    linkURL = ( $link.hostname + '/' + $link.pathname ).replace( '/\/$/', '' );

					// Check if the hash target is on this page.
					if ( pageURL === linkURL ) {
						// Deactivate all popups.
						if ( document.body.classList.contains( 'wengdo-has-popup-active' ) ) {
							deactivatePopup();
						}
					}
				}
			}
			document.addEventListener( 'click', handleHashLinkInsidePopup, false );
			document.addEventListener( 'touchend', handleHashLinkInsidePopup, false );
		},

		/**
		 * Function to init scroll to top.
		 */
		initScrollToTop: function() {
			var $scrollToTop = document.querySelector( '.wengdo-scroll-to-top' );

			if ( $scrollToTop ) {
				var handleScrollToTop = function( e ) {
					// Check target element.
					if ( ! e.target.closest( '.wengdo-scroll-to-top' ) ) return;

					e.preventDefault();

					var $link = e.target.closest( '.wengdo-scroll-to-top' ),
					    $target = document.getElementById( $link.getAttribute( 'href' ).replace( '#', '' ) );

					if ( $target ) {
						window.scrollTo({
							top: $target.getBoundingClientRect().top,
							behavior: 'smooth',
						});
					}
				}
				document.addEventListener( 'click', handleScrollToTop, false );
				document.addEventListener( 'touchend', handleScrollToTop, false );

				if ( $scrollToTop.classList.contains( 'wengdo-scroll-to-top-display-sticky' ) ) {
					var checkStickyOffset = function() {
						if ( window.pageYOffset > 0.5 * window.innerHeight ) {
							$scrollToTop.classList.add( 'sticky' );
						} else {
							$scrollToTop.classList.remove( 'sticky' );
						}
					}
					window.addEventListener( 'scroll', checkStickyOffset, false );
					checkStickyOffset();
				}
			}
		},

		/**
		 * Function that calls all init functions.
		 */
		initAll: function() {
			window.wengdo.initKeyboardAndMouseFocus();
			window.wengdo.initDropdownMenuReposition();
			window.wengdo.initMenuAccessibility();
			window.wengdo.initClickToggleDropdownMenu();
			window.wengdo.initDoubleTapMobileMenu();
			window.wengdo.initAccordionMenu();
			window.wengdo.initGlobalPopup();
			window.wengdo.initScrollToTop();
		},
	}

	document.addEventListener( 'DOMContentLoaded', window.wengdo.initAll, false );

})();

/* bolvo */
jQuery('p').each(function() {
    var $this = jQuery(this);
    if($this.html().replace(/\s|&nbsp;/g, '').length == 0)
        $this.remove();
});
//init Isotope
jQuery(window).load(function () {
    if ( jQuery('.grid_gallery_area').length ){
        jQuery(".isotopgrid").imagesLoaded(function () {
            var $grid = jQuery('.isotopgrid').isotope({
                itemSelector: '.element-item',
                layoutMode: 'masonry',
                 masonry: {
                  isFitWidth: true,
                    columnWidth: 200
                 }
            });
            // bind filter button click
            jQuery('.filters-button-group').on('click', '.isotopebutton', function () {
                var filterValue = jQuery(this).attr('data-filter');
                // use filterFn if matches value
                $grid.isotope({filter: filterValue});
            });
            // change is-checked class on buttons
            jQuery('.isotopebutton-group').each(function (i, buttonGroup) {
                var $buttonGroup = jQuery(buttonGroup);
                $buttonGroup.on('click', '.isotopebutton', function () {
                    $buttonGroup.find('.is-checked').removeClass('is-checked');
                    jQuery(this).addClass('is-checked');
                });
            });
        });
    }
});

// Four Item Carousel
if (jQuery('.four-item-carousel').length) {
    jQuery('.four-item-carousel').owlCarousel({
        //animateOut: 'slideOutDown',
        //animateIn: 'fadeIn',
        loop:true,
        margin:30,
        nav:true,
        singleItem:true,
        smartSpeed: 500,
        autoHeight: false,
        autoplay: true,
        autoplayTimeout:10000,
        navText: [ '<span class="fa fa-angle-left"></span>', '<span class="fa fa-angle-right"></span>' ],
        responsive:{
            0:{
                items:1
            },
            600:{
                items:2
            },
            800:{
                items:3
            },
            1024:{
                items:4
            },
            1200:{
                items:4
            },
        }
    });
}

;(function($) {
    "use strict";

	//Main Slider Carousel
	if ($('.main-slider-carousel').length) {
		$('.main-slider-carousel').owlCarousel({
			animateOut: 'fadeOut',
    		animateIn: 'fadeIn',
			loop:true,
			margin:0,
			nav:true,
			//autoHeight: true,
			smartSpeed: 500,
			autoplay: 6000,
			navText: [ '<span class="fa fa-angle-left"></span>', '<span class="fa fa-angle-right"></span>' ],
			responsive:{
				0:{
					items:1
				},
				600:{
					items:1
				},
				800:{
					items:1
				},
				1024:{
					items:1
				},
				1200:{
					items:1
				}
			}
		});
	}


	// Single Item Carousel
	if ($('.single-item-carousel').length) {
		$('.single-item-carousel').owlCarousel({
			//animateOut: 'slideOutDown',
    		//animateIn: 'fadeIn',
			loop:true,
			margin:0,
			nav:true,
			singleItem:true,
			smartSpeed: 500,
			autoHeight: false,
			autoplay: true,
			autoplayTimeout:10000,
			navText: [ '<span class="fa fa-angle-left"></span>', '<span class="fa fa-angle-right"></span>' ],
			responsive:{
				0:{
					items:1
				},
				600:{
					items:1
				},
				1024:{
					items:1
				},
			}
		});
	}



	// Two Item Carousel
	if ($('.two-item-carousel').length) {
		$('.two-item-carousel').owlCarousel({
			//animateOut: 'slideOutDown',
    		//animateIn: 'fadeIn',
			loop:true,
			margin:30,
			nav:true,
			singleItem:true,
			smartSpeed: 500,
			autoHeight: false,
			autoplay: true,
			autoplayTimeout:10000,
			navText: [ '<span class="fa fa-angle-left"></span>', '<span class="fa fa-angle-right"></span>' ],
			responsive:{
				0:{
					items:1
				},
				600:{
					items:1
				},
				800:{
					items:2
				},
				1024:{
					items:2
				},
				1200:{
					items:2
				}
			}
		});
	}



	// Gallery Carousel
	if ($('.gallery-carousel').length) {
		$('.gallery-carousel').owlCarousel({
			//animateOut: 'slideOutDown',
    		//animateIn: 'fadeIn',
			loop:true,
			margin:0,
			nav:true,
			singleItem:true,
			smartSpeed: 500,
			autoHeight: false,
			autoplay: true,
			autoplayTimeout:10000,
			navText: [ '<span class="fa fa-angle-left"></span>', '<span class="fa fa-angle-right"></span>' ],
			responsive:{
				0:{
					items:1
				},
				600:{
					items:2
				},
				800:{
					items:3
				},
				1024:{
					items:4
				},
				1200:{
					items:5
				},
			}
		});
	}


	// Four Item Carousel
	if ($('.four-item-carousel').length) {
		$('.four-item-carousel').owlCarousel({
			//animateOut: 'slideOutDown',
    		//animateIn: 'fadeIn',
			loop:true,
			margin:30,
			nav:true,
			singleItem:true,
			smartSpeed: 500,
			autoHeight: false,
			autoplay: true,
			autoplayTimeout:10000,
			navText: [ '<span class="fa fa-angle-left"></span>', '<span class="fa fa-angle-right"></span>' ],
			responsive:{
				0:{
					items:1
				},
				600:{
					items:2
				},
				800:{
					items:3
				},
				1024:{
					items:4
				},
				1200:{
					items:4
				},
			}
		});
	}


	//Fact Counter + Text Count
	if($('.count-box').length){
		$('.count-box').appear(function(){

			var $t = $(this),
				n = $t.find(".count-text").attr("data-stop"),
				r = parseInt($t.find(".count-text").attr("data-speed"), 10);

			if (!$t.hasClass("counted")) {
				$t.addClass("counted");
				$({
					countNum: $t.find(".count-text").text()
				}).animate({
					countNum: n
				}, {
					duration: r,
					easing: "linear",
					step: function() {
						$t.find(".count-text").text(Math.floor(this.countNum));
					},
					complete: function() {
						$t.find(".count-text").text(this.countNum);
					}
				});
			}

		},{accY: 0});
	}


	//Accordion Box
	if($('.accordion-box').length){
		$(".accordion-box").on('click', '.acc-btn', function() {

			var outerBox = $(this).parents('.accordion-box');
			var target = $(this).parents('.accordion');

			if($(this).hasClass('active')!==true){
				$(outerBox).find('.accordion .acc-btn').removeClass('active');
			}

			if ($(this).next('.acc-content').is(':visible')){
				return false;
			}else{
				$(this).addClass('active');
				$(outerBox).children('.accordion').removeClass('active-block');
				$(outerBox).find('.accordion').children('.acc-content').slideUp(300);
				target.addClass('active-block');
				$(this).next('.acc-content').slideDown(300);
			}
		});
	}



	//Product Tabs
	if($('.burgers-tab').length){
		$('.burgers-tab .burgers-tab-btns .p-tab-btn').on('click', function(e) {
			e.preventDefault();
			var target = $($(this).attr('data-tab'));

			if ($(target).hasClass('actve-tab')){
				return false;
			}else{
				$('.burgers-tab .burgers-tab-btns .p-tab-btn').removeClass('active-btn');
				$(this).addClass('active-btn');
				$('.burgers-tab .p-tabs-content .p-tab').removeClass('active-tab');
				$(target).addClass('active-tab');
			}
		});
	}


	// Burgers Carousel
	if ($('.burgers-carousel').length) {
		$('.burgers-carousel').owlCarousel({
			loop:true,
			margin:30,
			nav:true,
			smartSpeed: 700,
			autoplay: 5000,
			navText: [ '<span class="fa fa-angle-left"></span>', '<span class="fa fa-angle-right"></span>' ],
			responsive:{
				0:{
					items:1
				},
				600:{
					items:2
				},
				800:{
					items:3
				},
				1024:{
					items:3
				},
				1200:{
					items:3
				},
			}
		});
	}


	// Elements Animation
	if($('.wow').length){
		var wow = new WOW(
		  {
			boxClass:     'wow',      // animated element css class (default is wow)
			animateClass: 'animated', // animation css class (default is animated)
			offset:       0,          // distance to the element when triggering the animation (default is 0)
			mobile:       false,       // trigger animations on mobile devices (default is true)
			live:         true       // act on asynchronously loaded content (default is true)
		  }
		);
		wow.init();
	}

})(jQuery);